(* ****************************************************************************
 *                     DisplaySettingsChanger
 *                  Copyright (C) 2008 Fumio Kawamata
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; version 2 of the License.
 *
 *  This program is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANT-
 *  ABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
 *  Public License for more details.
 * ****************************************************************************
 *)
unit DisplaySettingsChanger_Main;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, Buttons, Menus, ImgList, ExtCtrls, ComCtrls;

type
  TFormMain = class(TForm)
    MainMenu: TMainMenu;
    MainMenuFile: TMenuItem;
    MainMenuFileExit: TMenuItem;
    MainMenuOption: TMenuItem;
    MainMenuOptionFont: TMenuItem;
    MainMenuHelp: TMenuItem;
    MainMenuHelpVersion: TMenuItem;
    StatusBar: TStatusBar;
    ImageList: TImageList;
    Label1: TLabel;
    ComboBoxFaultSize: TComboBox;
    Label2: TLabel;
    ComboBoxFaultColor: TComboBox;
    Label4: TLabel;
    ComboBoxFaultFreq: TComboBox;
    Label5: TLabel;
    Label6: TLabel;
    ComboBoxNewSize: TComboBox;
    ComboBoxNewColor: TComboBox;
    ComboBoxNewFreq: TComboBox;
    BitBtnSet: TBitBtn;
    BitBtnCancel: TBitBtn;
    MainMenuFileSet: TMenuItem;
    Memo: TMemo;
    FontDialog: TFontDialog;
    BitBtnStopCountDown: TBitBtn;
    Timer: TTimer;
    MainMenuFileStopCountDown: TMenuItem;
    procedure MainMenuHelpVersionClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure MainMenuFileExitClick(Sender: TObject);
    procedure MainMenuOptionFontClick(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure MainMenuFileSetClick(Sender: TObject);
    procedure TimerTimer(Sender: TObject);
    procedure MainMenuFileStopCountDownClick(Sender: TObject);
  private
    FaultWidth: Integer;
    FaultHeight: Integer;
    NewWidth: Integer;
    NewHeight: Integer;
    NewColor: Integer;
    NewFreq: Integer;
    TimeCounter: Integer;
    SettingIsChanged: Boolean;
    ProgramState: Integer; (* 0: Just started, 1: Stable state *)
    function ReadParametersFromIniFile(IniFileName: string): boolean;
    function WriteParametersToIniFile(IniFileName: string): boolean;
  public
  end;

var
  FormMain: TFormMain;

implementation

{$R *.dfm}

uses
  FKUtl, IniFiles;

var
  VersionInfo: FKUtl.TVersionInfo;
  IniFileFullPath: String;

const
  TimeUp=60; (* 60 seconds. *)
  StableTime=5; (* 5 seconds for stable. *)




(* *****************************************************************************
 * FormCreate
 * *****************************************************************************
 *)
procedure TFormMain.FormCreate(Sender: TObject);
var
  DevMode: tDeviceMode;
  I: Integer;
begin
  VersionInfo:=TVersionInfo.Create;
  VersionInfo.RetrieveItems;
  Caption:=VersionInfo.ProductName+' '+VersionInfo.FileVersion;

  IniFileFullPath:=ChangeFileExt(ParamStr(0), '.INI');
  FKUtl.ReadFormStatusFromIniFile(FormMain, IniFileFullPath);
  FKUtl.ReadFontInfoFromIniFile(FormMain, IniFileFullPath);
  ReadParametersFromIniFile(IniFileFullPath);

  I:=0;
  While EnumDisplaySettings(nil, I, DevMode) do begin
    With DevMode do begin
      Memo.Lines.Add(Format('%3d: %4d x %4d %2dbits %2dHz', [I,  dmPelsWidth, dmPelsHeight,
      dmBitsPerPel, dmDisplayFrequency]));
      inc(I);
    end;
  end;
  Memo.Lines.Add(Format('Current Desktop size: %d x %d', [Screen.Width, Screen.Height]));

  ProgramState:=0; (* Just started *)
  TimeCounter:=0;
  Timer.Interval:=1000; (* 1 second. *)
  Timer.Enabled:=True;
  SettingIsChanged:=False;
end;



(* *****************************************************************************
 * ReadParametersFromIniFile
 * *****************************************************************************
 *)
function TFormMain.ReadParametersFromIniFile(IniFileName: string): boolean;
var
  IniFile: TIniFile;
begin
  ReadParametersFromIniFile:=True;
  IniFile := TIniFile.Create(IniFileName);
  try
    FaultWidth:=IniFile.ReadInteger('General', 'FaultWidth', 800);
    FaultHeight:=IniFile.ReadInteger('General', 'FaultHeight', 600);
    NewWidth:=IniFile.ReadInteger('General', 'NewWidth', 1680);
    NewHeight:=IniFile.ReadInteger('General', 'NewHeight', 1050);
    NewColor:=IniFile.ReadInteger('General', 'NewColor', 32);
    NewFreq:=IniFile.ReadInteger('General', 'NewFreq', 60);
    ComboBoxFaultSize.Text:=IntToStr(FaultWidth)+' x '+IntToStr(FaultHeight);
    ComboBoxNewSize.Text:=IntToStr(NewWidth)+' x '+IntToStr(NewHeight);
    ComboBoxNewColor.Text:=IntToStr(NewColor);
    ComboBoxNewFreq.Text:=IntToStr(NewFreq);
  except
    ReadParametersFromIniFile:=False;
  end;
  IniFile.Free;
end;


(* *****************************************************************************
 * WriteParametersToIniFile
 * *****************************************************************************
 *)
function TFormMain.WriteParametersToIniFile(IniFileName: string): boolean;
var
  IniFile: TIniFile;
  S: String;
begin
  WriteParametersToIniFile:=True;
  IniFile := TIniFile.Create(IniFileName);
  try
    S:=ComboBoxFaultSize.Text;
    FaultWidth:=StrToIntDef(SplitBeforeDelimiter(S, 'x'), 800);
    FaultHeight:=StrToIntDef(S, 600);
    S:=ComboBoxNewSize.Text;
    NewWidth:=StrToIntDef(SplitBeforeDelimiter(S, 'x'), 1680);
    NewHeight:=StrToIntDef(S, 1050);
    NewColor:=StrToIntDef(ComboBoxNewColor.Text, 32);
    NewFreq:=StrToIntDef(ComboBoxNewFreq.Text, 60);
    IniFile.WriteInteger('General', 'FaultWidth', FaultWidth);
    IniFile.WriteInteger('General', 'FaultHeight', FaultHeight);
    IniFile.WriteInteger('General', 'NewWidth', NewWidth);
    IniFile.WriteInteger('General', 'NewHeight', NewHeight);
    IniFile.WriteInteger('General', 'NewColor', NewColor);
    IniFile.WriteInteger('General', 'NewFreq', NewFreq);
  except
    WriteParametersToIniFile:=False;
  end;
  IniFile.Free;
end;



(* *****************************************************************************
 * FormDestroy
 * *****************************************************************************
 *)
procedure TFormMain.FormDestroy(Sender: TObject);
begin
  FKUtl.WriteFormStatusToIniFile(FormMain, IniFileFullPath);
  FKUtl.WriteFontInfoToIniFile(FormMain, IniFileFullPath);
  WriteParametersToIniFile(IniFileFullPath);
end;



(* *****************************************************************************
 * MainMenuFileExitClick
 * *****************************************************************************
 *)
procedure TFormMain.MainMenuFileExitClick(Sender: TObject);
begin
  Close;
end;



(* *****************************************************************************
 * MainMenuFileOptionFontClick
 * *****************************************************************************
 *)
procedure TFormMain.MainMenuOptionFontClick(Sender: TObject);
var
  I: Integer;
begin
  FontDialog.Font:=Font;
  if FontDialog.Execute=True then
  begin
    Font:= FontDialog.Font;
    for I:= 0 to Screen.FormCount-1 do
    begin
      Screen.Forms[I].Font:=Font;
    end;
  end;
end;


(* *****************************************************************************
 * MainMenuFileHelpVersionClick
 * *****************************************************************************
 *)
procedure TFormMain.MainMenuHelpVersionClick(Sender: TObject);
begin
  VersionInfo.DisplayItems;
end;

procedure TFormMain.MainMenuFileSetClick(Sender: TObject);
var
  DeviceMode : TDeviceMode;
  Result: LongInt;
  S: String;
begin
  S:=ComboBoxNewSize.Text;
  NewWidth:=StrToIntDef(SplitBeforeDelimiter(S, 'x'), 1680);
  NewHeight:=StrToIntDef(S, 1050);
  ComboBoxNewSize.Text:=IntToStr(NewWidth)+' x '+IntToStr(NewHeight);

  EnumDisplaySettings(nil, 0, DeviceMode);
  DeviceMode.DmPelsWidth := NewWidth;
  DeviceMode.DmPelsHeight := NewHeight;
  DeviceMode.DmBitsPerPel:=NewColor;
  DeviceMode.DmDisplayFrequency:=NewFreq;
  DeviceMode.DmFields := DM_PELSWIDTH or DM_PELSHEIGHT or DM_BITSPERPEL or DM_DISPLAYFREQUENCY;
//ShowMessage('will execute changedisplaysettings.');
  Result:=ChangeDisplaySettings(DeviceMode, CDS_TEST);
//ShowMessage('Test Passed.');
  if Result<> DISP_CHANGE_SUCCESSFUL then
  begin
    MessageDlg('Test failed. Error Code: '+IntToStr(Result), mtWarning, [mbOK], 0);
    Exit;
  end;
  Result:=ChangeDisplaySettings(DeviceMode, CDS_UPDATEREGISTRY);
  if Result<> DISP_CHANGE_SUCCESSFUL then
  begin
    MessageDlg('The program passed test but failed registry update. Error Code: '+IntToStr(Result), mtWarning, [mbOK], 0);
    Exit;
  end;
//ShowMessage('Passed.');
(*
DISP_CHANGE_SUCCESSFUL = 0
DISP_CHANGE_RESTART = 1
DISP_CHANGE_FAILED = -1
DISP_CHANGE_BADMODE = -2
DISP_CHANGE_NOTUPDATED = -3
DISP_CHANGE_BADFLAGS = -4
DISP_CHANGE_BADPARAM = -5
*)
end;


(* *****************************************************************************
 * TimerTimer
 * *****************************************************************************
 *)
procedure TFormMain.TimerTimer(Sender: TObject);
var
  S: String;
  FaultWidth: Integer;
  FaultHeight: Integer;
begin
  Application.ProcessMessages;
  if ProgramState=0 then
  begin
    inc(TimeCounter);
    if TimeCounter>=StableTime then
    begin
      ProgramState:=1;
      TimeCounter:=0;
    end
    else begin
      StatusBar.SimpleText:='Count down for stable: '+IntToStr(StableTime-TimeCounter);
    end;
  end
  else if ProgramState=1 then
  begin
    S:=ComboBoxFaultSize.Text;
    FaultWidth:=StrToIntDef(SplitBeforeDelimiter(S, 'x'), 1);
    FaultHeight:=StrToIntDef(S, 1);
    if (Screen.Width=FaultWidth) and (Screen.Height=FaultHeight) then
    begin
      MainMenuFileSetClick(Sender);
      SettingIsChanged:=True;
    end;
    if (TimeCounter>=TimeUp) or (SettingIsChanged=True) then
    begin
      FormMain.Close;
    end;
    inc(TimeCounter);
    StatusBar.SimpleText:='Count down for forced terminate: '+IntToStr(TimeUp-TimeCounter);
  end;
end;

procedure TFormMain.MainMenuFileStopCountDownClick(Sender: TObject);
begin
  Timer.Enabled:=False;
  StatusBar.SimpleText:='';
  BitBtnStopCountDown.Enabled:=False;
end;

end.

